import sys
import os
import importlib.metadata
import argparse

from . import meta


def main():

    assert 'NAME' not in globals()

    exec(meta.get_meta_str())

    assert 'NAME' in globals()


    this_file_path = __file__

    assert '__main__.py' == os.path.basename(this_file_path)

    package_module_dir_path = os.path.dirname(this_file_path)


    metadata = None
    try:
        metadata = importlib.metadata.metadata(NAME)
    except importlib.metadata.PackageNotFoundError:
        pass

    name    = metadata['Name']    if metadata else 'name_not_available'
    summary = metadata['Summary'] if metadata else 'summary_not_available'
    version = metadata['Version'] if metadata else 'version_not_available'


    arg_parser = argparse.ArgumentParser(
        prog                  = name,
        description           = summary,
        formatter_class       = argparse.ArgumentDefaultsHelpFormatter,
        fromfile_prefix_chars = '@'
    )
    
    arg_parser.add_argument(
        '-V', '--version',
        action  = 'version',
        version = f'{name} {version}'
    )
    
    args = arg_parser.parse_args()


    data_dir_path = os.path.join(package_module_dir_path, 'data')

    assert os.path.isdir(data_dir_path)

    quote_file_path = os.path.join(data_dir_path, 'hello.txt')

    assert os.path.isfile(quote_file_path)

    with open(quote_file_path, 'r') as quote_file:

        quote_str = quote_file.read().strip()

    print(quote_str)
    
    return 0


if '__main__' == __name__:
    sys.exit(main())





import os


def get_meta_str() -> str:

    this_file_path = __file__

    assert os.path.isfile(this_file_path)

    package_module_dir_path = os.path.dirname(this_file_path)

    assert os.path.isdir(package_module_dir_path)

    meta_sh_file_path = os.path.join(package_module_dir_path, 'meta.sh')

    assert os.path.isfile(meta_sh_file_path)

    pipefd = os.pipe()

    pid = os.fork()

    if 0 == pid:

        os.close(pipefd[0])

        os.dup2(pipefd[1], 1)

        os.close(pipefd[1])

        argv = [meta_sh_file_path]

        os.execve(argv[0], argv, os.environ)

        assert False, 'os.execve(...) returned, which is unexpected'

    assert 0 < pid

    os.close(pipefd[1])

    wait_pid, retcode_raw = os.wait()

    assert wait_pid == pid

    retcode = retcode_raw >> 8

    assert 0 == retcode

    meta_bytes = b''
    while True:
        byte = os.read(pipefd[0], 1)
        if 1 == len(byte):
            meta_bytes += byte
        else:
            assert 0 == len(byte)
            break

    meta_str = meta_bytes.decode('utf-8')

    return meta_str



